@@ -8,7 +8,7 @@ use strict;
 
 use POE::Kernel; # for MakeMaker
 use vars qw($VERSION);
-$VERSION = do {my@r=(0,q$Rev: 23 $=~/\d+/g);sprintf"%d."."%04d"x$#r,@r};
+$VERSION = do {my@r=(0,q$Rev: 30 $=~/\d+/g);sprintf"%d."."%04d"x$#r,@r};
 
 # Include common signal handling.
 use POE::Loop::GlibCommon;
@@ -43,21 +43,38 @@ sub loop_attach_uidestroy {
       }
     );
 }
+
 sub loop_initialize {
   my $self = shift;
 
-  $glib_mainloop = Glib::MainLoop->new unless ($Glib::main_depth > 0);
+  $glib_mainloop = Glib::MainLoop->new unless (Glib::main_depth() > 0);
+  Glib->install_exception_handler (\&ex);
 
 }
 
 sub loop_run {
   (defined $glib_mainloop) && $glib_mainloop->run;
+  if (defined $POE::Kernel::_glib_loop_exception) {
+	my $ex = $POE::Kernel::_glib_loop_exception;
+	undef $POE::Kernel::_glib_loop_exception;
+  	die $ex;
+  }
 }
 
 sub loop_halt {
   (defined $glib_mainloop) && $glib_mainloop->quit;
 }
 
+our $_glib_loop_exception;
+
+sub ex {
+  $_glib_loop_exception = shift;
+  &loop_finalize;
+  &loop_halt;
+
+  return 0;
+}
+
 1;
 
 __END__
@@ -85,4 +102,10 @@ L<POE>, L<POE::Loop>, L<Glib>, L<Glib::MainLoop>
 Please see L<POE> for more information about authors, contributors,
 and POE's licensing.
 
+=for poe_tests
+
+{
+	module => 'Glib',
+}
+
 =cut
